#include <windows.h>
//#include <pp_crt.h>
#include "filters.h"
#include <vorbis/vorbisfile.h>


static size_t read_func(void* ptr,size_t size,size_t nmemb,HANDLE src)
{
	DWORD br=0;
	ReadFile(src,ptr,size*nmemb,&br,0);
	return br;
}

static int seek_func(HANDLE f,ogg_int64_t offset,int whence)
{
	SetFilePointer(f,(DWORD)offset,(long*)&offset+1,whence);
	return 0;
}

static int close_func(HANDLE f) {CloseHandle(f);return 0;};

static long tell_func(HANDLE f)
{
	return SetFilePointer(f,0,0,FILE_CURRENT);
}

/*
typedef struct {
  size_t (*read_func)  (void *ptr, size_t size, size_t nmemb, void *datasource);
  int    (*seek_func)  (void *datasource, ogg_int64_t offset, int whence);
  int    (*close_func) (void *datasource);
  long   (*tell_func)  (void *datasource);
} ov_callbacks;
*/

static void * callbacks[4]=
{
	read_func,seek_func,close_func,tell_func
};


#define BUFSIZE 0x4000

typedef struct
{
	OggVorbis_File vf;
	UINT bpos,data;
	char buf[BUFSIZE];
} INPUT;

_declspec(dllexport) INPUT* PASCAL OpenFilterInput( LPSTR fn, long *lSamprate, WORD *wBitsPerSample, WORD *wChannels, HWND hWnd, long *lChunkSize)
{
	HANDLE f;
	INPUT * i;
	f=CreateFile(fn,GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,0,0);
	if (f==INVALID_HANDLE_VALUE) return 0;
	i=(INPUT*)LocalAlloc(LMEM_FIXED|LMEM_ZEROINIT,sizeof(INPUT));
	if (!i) return 0;
	if (ov_open_callbacks(f,&i->vf,0,0,*(ov_callbacks*)&callbacks))
	{
		CloseHandle(f);
		LocalFree(i);
		return 0;
	}
	{
		vorbis_info* vi=ov_info(&i->vf,-1);
		*lSamprate=vi->rate;
		*wBitsPerSample=16;
		*wChannels=vi->channels;
		*lChunkSize=0x1000;
	}
	return i;
}

_declspec(dllexport) void PASCAL CloseFilterInput(INPUT* i)
{
	ov_clear(&i->vf);
	LocalFree(i);
}

_declspec(dllexport) DWORD PASCAL ReadFilterInput(INPUT* i, BYTE *out, long bytes)
{
	UINT wr=0;
	if (bytes)
	{
		while(1)
		{
			UINT d;
			if (!i->data)
			{
				i->data=ov_read(&i->vf,i->buf,BUFSIZE,0,2,1,0);
				if ((signed)i->data<=0) {i->data=0;break;}
				i->bpos=0;
			}
			d=bytes-wr;
			if (d>i->data) d=i->data;
			memcpy(out,i->buf+i->bpos,d);
			out+=d;
			wr+=d;
			i->bpos+=d;
			i->data-=d;
			if (wr>=bytes) break;
		}
	}
	return wr;
}

_declspec(dllexport) DWORD PASCAL FilterOptions(INPUT* i)
{
	return 0;
}	    

_declspec(dllexport) long PASCAL FilterGetFileSize(INPUT* i)
{
	double s=ov_time_total(&i->vf,-1);
	if (s==OV_EINVAL) return 0;
	else
	{
		vorbis_info* vi=ov_info(&i->vf,-1);
		return (int)(s*vi->rate*vi->channels*2);
	}
}

_declspec(dllexport) DWORD PASCAL FilterOptionsString(INPUT* i,char* s)
{
	vorbis_info* vi=ov_info(&i->vf,-1);
	wsprintf(s,"average bitrate: %u kbps\nsample rate: %u Hz, channels: %u",vi->bitrate_nominal/1000,vi->rate,vi->channels);
	return 0;
}

_declspec(dllexport) BOOL PASCAL FilterUnderstandsFormat(LPSTR filename)
{
	char* p=strrchr(filename,'.');
	if (p) return !stricmp(p,".ogg");
	return 0;
}
